import 'dart:ui';
import 'package:flutter/material.dart';

enum NUIThemeStyle{
  LIGHT,
  DARK
}

class NUIThemePair<T>{
  NUIThemePair({required this.light, required this.dark});
  T light;
  T dark;
}

class NUIThemeData{
  NUIThemeData({
    required this.fontFamily,
    required this.primaryColor,
    required this.primarySwatch,
    required this.canvasColor,
    required this.backgroundColor,
    required this.accentColor,
    required this.textTheme,
    required this.colors,
    this.colorScheme,
    this.defaultTheme = NUIThemeStyle.LIGHT
  });
  String fontFamily;
  NUIThemePair<Color> primaryColor;
  NUIThemePair<Color> canvasColor;
  NUIThemePair<Color> backgroundColor;
  NUIThemePair<Color> accentColor;
  NUIThemePair<MaterialColor> primarySwatch;
  NUIThemePair<TextTheme> textTheme;
  List<NUIThemePair<Color>> colors;
  NUIThemeStyle defaultTheme;
  ColorScheme? colorScheme;

  ThemeData getTheme(BuildContext context){
    // final isLightTheme = defaultTheme == null ? isOnLightTheme(context) : defaultTheme == NUIThemeStyle.LIGHT;
    final isLightTheme = true;
    return ThemeData(
      brightness: isLightTheme != false ? Brightness.light : Brightness.dark,
      primaryColor: getByTheme(context, primaryColor, isLight: isLightTheme),
      canvasColor: getByTheme(context, canvasColor, isLight: isLightTheme),
      primarySwatch: getByTheme(context, primarySwatch, isLight: isLightTheme),
      textTheme: getByTheme(context, textTheme, isLight: isLightTheme),
      fontFamily: fontFamily,
      colorScheme: colorScheme ?? ColorScheme(
          primary: getByTheme(context, primaryColor, isLight: isLightTheme),
          secondary: getByTheme(context, primaryColor, isLight: isLightTheme),
          surface: getByTheme(context, backgroundColor, isLight: isLightTheme),
          background: getByTheme(context, primaryColor, isLight: isLightTheme),
          error: getByTheme(context, accentColor, isLight: isLightTheme),
          onPrimary: getByTheme(context, primaryColor, isLight: isLightTheme),
          onSecondary: getByTheme(context, primaryColor, isLight: isLightTheme),
          onSurface: getByTheme(context, primaryColor, isLight: isLightTheme),
          onBackground: getByTheme(context, primaryColor, isLight: isLightTheme),
          onError: getByTheme(context, accentColor, isLight: isLightTheme),
          brightness: isLightTheme != false ? Brightness.light : Brightness.dark
      )
    );
  }

  T getByTheme<T>(BuildContext context, NUIThemePair<T> pair, {bool isLight = true}){
    // final isLightTheme = isLight ?? isOnLightTheme(context);
    final isLightTheme = true;
    if(isLightTheme != false) return pair.light;
    return pair.dark;
  }

  TextStyle getFont(BuildContext context, {double size = 14, NUIThemePair<Color>? color, Color? singleColor, String? fontName, bool isBold = false, Shadow? shadow, FontWeight? fontWeight, TextDecoration decoration = TextDecoration.none, bool isItalic = false, List<FontFeature>? fontFeatures}){
    return TextStyle(fontSize: size, color: color != null ? getByTheme<Color>(context, color) : singleColor, fontWeight: fontWeight ?? (isBold ? FontWeight.bold : FontWeight.normal), shadows: shadow != null ? [shadow] : null, decoration: decoration, fontFamily: fontName ?? fontFamily, fontStyle: isItalic ? FontStyle.italic : FontStyle.normal, fontFeatures: fontFeatures);
  }

  Color getColor(BuildContext context, NUIThemePair<Color> color){
    return getByTheme<Color>(context, color);
  }
}